<?php
namespace PlaNext\Services;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\SMTP;
use Teknisa\Libs\Util\Utilities as TekLibUtil;

class Utilities{
    protected $crudService;
    protected $environment;
    protected $tmpdir = '/tmp/';
    protected $tempFiles = [];
    
    public function __construct($crudService, $environment) {
        $this->crudService = $crudService;
        $this->environment = $environment;
    }

    public function getDatasourceData($dataSourceName, $params=[]){
	    $conditions = [];
	    
	    if(is_array($params)){
	        foreach($params as $key=>$value){
	            $condition = [
	                'columnName' => $key,
    	            'operator' => '=',
    	            'value' => $value    
	            ];
	            array_push($conditions, $condition); 
	        }
	    }
	    $result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
	    return $result;
    }
    
    public function getDatasourceFirstRow($dataSourceName, $params=[]){
	    $data = $this->getDatasourceData($dataSourceName, $params);
	    if (count($data) > 0)
	        return $data[0];
	    return null;
    }
    
    public function getRowValue($row, $columnName) {
        if(isset($row)){
            if(isset($row[$columnName]))
                return $row[$columnName];
        }
            
        return null;
    }
    
    public function getFirstRowValue($dataSourceName, $columnName, $params=[]) {
        $row = $this->getDatasourceFirstRow($dataSourceName, $params);
        if(isset($row)){
            return $this->getRowValue($row, $columnName);
        }
        return null;
    }
    
    public function getInitialValues(Request $req, Response $res){
        $cdfilial = $this->environment->getCdFilial();
        $dtprocessa = $this->getFirstRowValue('dtprocessa', 'DTPROCESSA', ['CDFILIAL' => $cdfilial]);
        $plateProdInfos = $this->getDatasourceFirstRow('plateProdInfos');
        $data = array(
            'NRORG' => $this->environment->getNrOrg(),
            'CDOPERADOR' => $this->environment->getCdOperador(),
            "CDFILIAL" => $this->environment->getCdFilial(),
            "DTPROCESSA" => $dtprocessa,
            "CDPICTPRAT" => $this->getRowValue($plateProdInfos, "CDPICTPRAT"),
            "CDPICTPROD" => $this->getRowValue($plateProdInfos, "CDPICTPROD"),
            "CDNVPRATO" => $this->getRowValue($plateProdInfos, "CDNVPRATO")
        );
        $res->addDataSet(new DataSet("initialValues", $data));
    }

    public function getMailSender($cdfilial) {
        // search FILIAL SMTP params
        $smtpParams = $this->getDatasourceFirstRow('smtpUnit', ['CDFILIAL' => $cdfilial]);
        if($smtpParams == null) {
            throw new \Exception("Utilities::NO_SMTP_UNIT");
        }
        if(!isset($smtpParams['DSSMTPEMAILF'])) {
            throw new \Exception("Utilities::NO_SMTP_UNIT_HOST");
        }
        $host = null;
        $port = null;
        if(isset($smtpParams['DSSMTPEMAILF'])) {
            $parts = explode(':', $smtpParams['DSSMTPEMAILF']);;
            $host = $parts[0];
            $port = '25';
            if(count($parts) > 1) {
                $port = $parts[1];
            }
        }

        $username = $smtpParams['DSLOGINSMTP'];
        // $password = TekLibUtil::getDelphiPasswordEncrypted($smtpParams['CDSENHEMAILF'], false);
        $password = $smtpParams['CDSENHEMAILF'];
        $password = $this->decrypt($password, 2);
        $smtpSecure = '';
        $smtpAuth = ($username && $password);
        if ($smtpParams['IDUTTLSEMAIL'] == 'S') {
            $smtpSecure = 'tls';
        }
        if ($smtpParams['IDUTSSLEMAIL'] == 'S') {
            $smtpSecure = 'ssl';
        }
        $from = $smtpParams['DSLOGINSMTP'];
        $fromName = $smtpParams['DSLOGINSMTP'];
        return new SMTP([
            'host' => $host,
            'port' => $port,
            'username' => $username,
            'password' => $password,
            'smtpAuth' => $smtpAuth,
            'smtpSecure' => $smtpSecure,
            'from' => $from,
            'fromName' => $fromName
        ]);
    }
    
    public function getEmailMessageFromTemplate($templateName, $params = []){
        $templatePath = __DIR__ . '/../EmailTemplates/';
        $templateFileName = $templatePath . $templateName . '.html';
        if(!file_exists($templateFileName)) {
            throw \Exception("Utilities::NO_EMAIL_TEMPLATE");
        }
        $templateContent = file_get_contents($templateFileName);
        if(is_array($params)) {
            foreach($params as $key => $value) {
                if (is_string($value)) {
                    $templateContent = str_replace('<>' . $key . '<>', $value, $templateContent);
                }
            }
        }
        return $templateContent;
    }
    
    private function createTempFile($fileNameWithExtension, $fileContents){
        try {
            $tempFileName = $this->$tmpdir . $fileNameWithExtension;
            $this->tempFiles[] = $tempFileName;
            return $tempFileName;
        } catch (\Exception $e) {
            throw new \Exception("Utilities::CANT_CREATE_FILE" . '###' . $fileNameWithExtension);
        }
    }
    
    private function deleteTempFiles() {
        foreach($this->tempFiles as $file) {
            unlink($file);
        } 
    }
    
    public function sendMail($cdfilial, $to, $subject, $message, $attachments=[], $fromName=null){
        try {
            $mailSender = $this->getMailSender($cdfilial);
            $mailSender->setSubject($subject);
            if($fromName) {
                $mailSender->setFromName($fromName);
            }
            $mailSender->clearAttachments();
            if(is_array($attachments)) {
                
                foreach($attachments as $fileNameWithExtension => $fileContents) {
                    $tmpFileName = $this->createTempFile($fileNameWithExtension, $fileContents);
                    $mailSender->addAttachment($tmpFileName, $fileNameWithExtension);
                }
            }
            $mailSender->sendMail($message, $to);
        } finally{
            $this->deleteTempFiles(); 
        }
    }
    
    /*REMOVER AO SAIR A TAREFA DA LIB - ISSUE 491615*/
    
    private function bround($num) {
        if (strpos($num, '.') === false)
            return $num;

        $num = $num;
        $num_arr = explode('.', $num);
        if (empty($num_arr[1])) return $num;
        if($num_arr[1] == 5) {
            if($num_arr[0] % 2) return $num_arr[0]+1;
            else return $num_arr[0];
        } else return round($num);
    }

    private function encrypt($stString, $inOffset = 2) {
        $inTamanho = intval(strlen($stString));
        $inMeioAnt = intval($inTamanho/2);
        $inMeioPos = intval(self::bround(($inTamanho/2) + 0.1) + 1);

        $stVetor = $stString;

        for($byAux = 1; $byAux <= $inMeioAnt; $byAux++) {
            $chAux = $stVetor[$byAux-1];
            $stVetor[$byAux-1] = $stVetor[$byAux*2-1];
            $stVetor[$byAux*2-1] = $chAux;
        }
        
        for($byAux = $inTamanho; $byAux >= $inMeioPos; $byAux--) {
            $chAux = $stVetor[$byAux*2 - ($inTamanho + 1)-1];
            $stVetor[$byAux*2 - ($inTamanho + 1)-1] = $stVetor[$byAux-1];
            $stVetor[$byAux-1] = $chAux;
        }

        $stString = '';

        for($byAux = 1; $byAux <= $inTamanho; $byAux++) {
            $stString = $stString.(chr(ord($stVetor[$byAux-1]) + $inOffset));
        }

        return $stString;
    }

    private function decrypt($stString, $inOffset = 2) {
        if($stString == null || $stString == ""){
            return null;
        }
        
        $inTamanho = intval(strlen($stString));
        $inMeioAnt = intval($inTamanho/2); 
        $inMeioPos = intval(self::bround(($inTamanho/2) + 0.1) + 1);

        $stVetor = $stString;

        for($byAux = $inMeioPos; $byAux <= $inTamanho; $byAux++) {
            $chAux = $stVetor[intval($byAux*2 - ($inTamanho + 1))-1];
            $stVetor[intval($byAux*2 - ($inTamanho + 1))-1] = $stVetor[intval($byAux)-1];
            $stVetor[intval($byAux)-1] = $chAux;
        }

        for($byAux = $inMeioAnt; $byAux >= 0; $byAux--) {
            $chAux = $stVetor[$byAux-1];
            $stVetor[$byAux-1] = $stVetor[$byAux*2-1];
            $stVetor[$byAux*2-1] = $chAux;
        }
        
        $stString = '';

        for($byAux = 1; $byAux <= $inTamanho; $byAux++) {
            $stString = $stString.(chr(ord($stVetor[$byAux-1]) - $inOffset));
        }

        return $stString;
    }
    
    /*REMOVER AO SAIR A TAREFA DA LIB - ISSUE 491615*/
}